/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

/**
 * @file myx_gc_canvas.cpp 
 * @brief Generic canvas main class and entry point.
 * 
 */

#ifdef _WINDOWS
  #include <windows.h>
#else
#endif // ifdef _WINDOWS

//#include <libxml/xmlmemory.h>
//#include <libxml/parser.h>
#include <stdlib.h>
#include <assert.h>
#include "myx_xml_aux_functions.h"
#include <gl/gl.h>
#include <string>
#include "..\include\myx_gc_canvas.h"

//----------------------------------------------------------------------------------------------------------------------

// Factory function for a canvas.
GENERIC_CANVAS_API CGenericCanvas* CreateGenericCanvas(GCContext Context)
{
	return new CGenericCanvas(Context);
}

//----------------------------------------------------------------------------------------------------------------------

CGenericCanvas::CGenericCanvas(GCContext Context)
{ 
	FContext = Context;
  FBackgroundColor = 0xFFFFFFFF;
  FViewport.Left = 0;
  FViewport.Top = 0;
  FViewport.Width = 100;
  FViewport.Height = 100;
}

//----------------------------------------------------------------------------------------------------------------------

void CGenericCanvas::Release(void)
{
  delete this;
}

//----------------------------------------------------------------------------------------------------------------------

void CGenericCanvas::SetBackgroundColor(COLORREF NewColor)
{
  if (NewColor != FBackgroundColor)
  {
    FBackgroundColor = NewColor;
    ApplyBackground();
  }
}

//----------------------------------------------------------------------------------------------------------------------

void CGenericCanvas::ApplyBackground()
{
  glClearColor(GetRValue(FBackgroundColor), GetGValue(FBackgroundColor), GetBValue(FBackgroundColor), 1);
}

//----------------------------------------------------------------------------------------------------------------------

void CGenericCanvas::SetViewport(PVIEWPORT NewViewport)
{
  FViewport = *NewViewport;
  ApplyViewport();
}

//----------------------------------------------------------------------------------------------------------------------

void CGenericCanvas::ApplyViewport()
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  //gluPerspective(35, Width / Height, 1, 100);

  glOrtho(FViewport.Left, FViewport.Left + FViewport.Width, FViewport.Top + FViewport.Height, FViewport.Top, -100, 100);
  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, FViewport.Width, FViewport.Height);
}

//----------------------------------------------------------------------------------------------------------------------

void CGenericCanvas::ClearBuffers()
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

//----------------------------------------------------------------------------------------------------------------------

void CGenericCanvas::Render()
{
  ClearBuffers();
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
}

//----------------------------------------------------------------------------------------------------------------------

// Reads the layout info stored in the given (XML) file and creates a list of figures.
GC_ERROR CGenericCanvas::AddFiguresFromFile(char* FileName)
{
  GC_ERROR result = GC_NO_ERROR;

  xmlDocPtr doc;
  xmlNodePtr root,cur;
  MYX_APPLICATION_OPTIONS *app_options;
  MYX_OPTION_GROUP * group;

  *error_code= MYX_NO_ERROR;

  if (! file_exists(filename) )
  {
    *error_code= MYX_ERROR_CANT_OPEN_FILE;
    return NULL;
  }

  doc= myx_xmlParseFile(filename);

  if (doc == NULL)
  {
    *error_code= MYX_XML_PARSE_ERROR;
    return NULL;
  }

  root= xmlDocGetRootElement(doc);

  if (root == NULL)
  {
    *error_code= MYX_XML_EMPTY_DOCUMENT;
    xmlFreeDoc(doc);
    return NULL;
  }
  
  if (xmlStrcmp(root->name, (const xmlChar *) "application_options"))
  {
    *error_code= MYX_XML_NO_VALID_DOCUMENT;
    xmlFreeDoc(doc);
    return NULL;
  }

  app_options= g_malloc(sizeof(MYX_APPLICATION_OPTIONS));

  /*count the number of group_elements*/
  app_options->option_groups_num= get_child_count(root,"group");

  app_options->option_groups= 
    app_options->option_groups_num==0 ? NULL :
    g_malloc(sizeof(MYX_OPTION_GROUP) * app_options->option_groups_num);

  /*read each group-element together with its name_value_pairs*/
  for (group= app_options->option_groups, cur= root->children; cur != NULL;
       cur= cur->next)
  {
    if (xmlStrcmp(cur->name, "group"))
      continue;

    group->name= xmlGetProp(cur, "name");
    get_name_value_pairs(cur, group);
    group++;
  }

  xmlFreeDoc(doc);

  return result;
}

//----------------------------------------------------------------------------------------------------------------------

