package com.mysql.grt;

import junit.framework.TestCase;
import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestSuite;
import com.mysql.grt.db.Column;
import com.mysql.grt.db.ColumnList;
import com.mysql.grt.db.DistinctDatatype;
import com.mysql.grt.modules.GrtBase;

/**
 * Test of the Grt main class
 * 
 * @author Mike
 * @version 1.0, 11/29/04
 */
public class GrtTest extends TestCase {

	public GrtTest(String name) {
		super(name);
	}

	/**
	 * main function so the test can be executed
	 */
	public static void main(String[] args) {
		junit.textui.TestRunner.run(GrtTest.class);
	}

	/**
	 * The suite function builds the testsuite
	 */
	public static Test suite() {
		TestSuite suite = new TestSuite();
		suite.addTest(new GrtTest("testPrepareGrtXml"));
		suite.addTest(new GrtTest("testGetObjectsFromGrtXml"));
		suite.addTest(new GrtTest("testCallModuleFunctionNonExistingFunction"));
		suite.addTest(new GrtTest("testException"));
		return suite;
	}

	/**
	 * Tests the prepareGrtXml function
	 */
	public void testPrepareGrtXml() {
		String grtXml = "<value type=\"string\">test</value>";
		String xml = Grt.prepareGrtXml(grtXml);

		Assert.assertEquals(xml, "<?xml version=\"1.0\"?>\n<data>\n" + grtXml
				+ "</data>\n");
	}

	/**
	 * Tests XML parsing and generation
	 */
	public void testGetObjectsFromGrtXml() {
		String grtXml = "<value type=\"dict\">\n"
				+ "<value type=\"list\" content-type=\"dict\" content-struct-name=\"db.Column\" key=\"columns\">\n"
				+ "<value type=\"dict\" struct-name=\"db.Column\">\n"
				+ "<value type=\"string\" key=\"name\">Host</value>\n"
				+ "<value type=\"string\" key=\"_id\">328758:2732734:32434234</value>\n"
				+ "<value type=\"int\" key=\"precision\">0</value>\n"
				+ "<value type=\"int\" key=\"scale\">0</value>\n"
				+ "<value type=\"int\" key=\"isNullable\">1</value>\n"
				+ "<value type=\"int\" key=\"length\">30</value>\n"
				+ "<value type=\"string\" key=\"datatypeName\">varchar</value>\n"
				+ "<value type=\"string\" key=\"defaultValue\">testing</value>\n"
				+ "<value type=\"string\" key=\"characterSetName\">default</value>\n"
				+ "<value type=\"string\" key=\"collationName\">default</value>\n"
				+ "<value type=\"dict\" struct-name=\"db.DistinctDatatype\" key=\"sqlDistinctType\">\n"
				+ "<value type=\"string\" key=\"name\">HostDistDatatype</value>\n"
				+ "<value type=\"string\" key=\"_id\">423423:89734566:746345234</value>\n"
				+ "<value type=\"int\" key=\"length\">30</value>\n"
				+ "<value type=\"int\" key=\"precision\">0</value>\n"
				+ "<value type=\"int\" key=\"scale\">0</value>\n"
				+ "<value type=\"string\" option=\"ref\" key=\"sqlSimpleType\">972612:7234532:196457312</value>\n"
				+ "</value>\n" + "</value>\n" + "</value>\n" + "</value>\n";
		String xml = Grt.prepareGrtXml(grtXml);

		GrtHashMap root = (GrtHashMap) Grt.getObjectsFromGrtXml(xml);
		Assert.assertNotNull(root);

		Object list = root.getObject("columns");

		Assert.assertNotNull(list);
		Assert.assertEquals(list.getClass().getName(),
				"com.mysql.grt.db.ColumnList");

		ColumnList columnList = (ColumnList) list;

		Column column = columnList.get(0);

		Assert.assertEquals(column.getDatatypeName(), "varchar");

		DistinctDatatype distDatatype = column.getSqlDistinctType();

		Assert.assertEquals(distDatatype.getLength(), 30);

		Assert.assertEquals(Grt.prepareGrtXml(root.getGrtXml("")), xml);
	}

	/**
	 * Tests the callModuleFunction function by calling a non-existing function
	 */
	public void testCallModuleFunctionNonExistingFunction() {
		String xml = Grt.callModuleFunction(GrtBase.class,
				"nonExistingFuntion", "", "");

		GrtHashMap result = (GrtHashMap) Grt.getObjectsFromGrtXml(xml);

		Assert.assertNotNull(result.getObject("error"));
		Assert.assertNotNull(result.getObject("stack"));
	}
	
	public void testException() {
		String xml = Grt.callModuleFunction(GrtBase.class, "throwException",
					"", "");
		
		GrtHashMap result = (GrtHashMap) Grt.getObjectsFromGrtXml(xml);
		
		Assert.assertNotNull(result.getObject("error"));
		Assert.assertNotNull(result.getObject("stack"));
		Assert.assertEquals(result.getObject("error"), "Exception Test");
	}
}